@@ -20,20 +20,26 @@
 
 #include "frame_glom.h"
 #include "application.h"
-#include "appstate.h"
+#include <glom/libglom/appstate.h>
 #include "mode_design/users/dialog_groups_list.h"
 #include "dialog_database_preferences.h"
 #include "reports/dialog_layout_report.h"
-#include "utils.h"
-#include "data_structure/glomconversions.h"
-#include "data_structure/layout/report_parts/layoutitem_summary.h"
-#include "data_structure/layout/report_parts/layoutitem_fieldsummary.h"
+#include <glom/libglom/utils.h>
+#include <glom/libglom/data_structure/glomconversions.h>
+#include <glom/libglom/data_structure/layout/report_parts/layoutitem_summary.h>
+#include <glom/libglom/data_structure/layout/report_parts/layoutitem_fieldsummary.h>
+#include <glom/reports/report_builder.h>
+#include <glom/mode_design/dialog_add_related_table.h>
 #include "relationships_overview/dialog_relationships_overview.h"
 #include "filechooser_export.h"
+#include <glom/glom_privs.h>
 #include <sstream> //For stringstream.
 #include <fstream>
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
 : PlaceHolder(cobject, refGlade),
   m_pLabel_Name(0),
@@ -54,6 +60,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   m_pDialog_Reports(0),
   m_pDialog_Fields(0),
   m_pDialog_Relationships(0),
+  m_dialog_addrelatedtable(0),
   m_pDialogConnection(0),
   m_pDialogConnectionFailed(0),
   m_pDialogLayoutReport(0)
@@ -145,18 +152,6 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
     std::cerr << ex.what() << std::endl;
   }
 
-  try
-  {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_design");
-
-    refXml->get_widget_derived("window_design", m_pDialog_Fields);
-    m_pDialog_Fields->signal_hide().connect( sigc::mem_fun(*this, &Frame_Glom::on_developer_dialog_hide));
-  }
-  catch(const Gnome::Glade::XmlError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
-  }
-
   try
   {
     Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_design");
@@ -213,7 +208,6 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   //This means that set_document and load/save are delegated to these children:
   add_view(m_pBox_Tables);
   add_view(m_pBox_Reports);
-  add_view(m_pDialog_Fields); //Also a composite view.
   add_view(m_pDialog_Relationships); //Also a composite view.
   add_view(m_pDialogConnection); //Also a composite view.
   add_view(&m_Notebook_Data); //Also a composite view.
@@ -274,6 +268,13 @@ Frame_Glom::~Frame_Glom()
     delete m_pDialog_Fields;
     m_pDialog_Fields = 0;
   }
+
+  if(m_dialog_addrelatedtable)
+  {
+    remove_view(m_dialog_addrelatedtable);
+    delete m_dialog_addrelatedtable;
+    m_dialog_addrelatedtable = 0;
+  }
 }
 
 void Frame_Glom::set_databases_selected(const Glib::ustring& strName)
@@ -380,7 +381,7 @@ void Frame_Glom::show_table_refresh()
   show_table(m_table_name);
 }
 
-void Frame_Glom::show_table(const Glib::ustring& table_name)
+void Frame_Glom::show_table(const Glib::ustring& table_name, const Gnome::Gda::Value& primary_key_value_for_details)
 {
   App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
 
@@ -411,7 +412,21 @@ void Frame_Glom::show_table(const Glib::ustring& table_name)
       case(MODE_Data):
       {
         strMode = _("Data");
-        m_Notebook_Data.init_db_details(m_table_name);
+        FoundSet found_set;
+        found_set.m_table_name = m_table_name;
+
+        //Sort by the ID, just so we sort by something, so that the order is predictable:
+        sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
+        if(field_primary_key)
+        {
+          sharedptr<LayoutItem_Field> layout_item_sort = sharedptr<LayoutItem_Field>::create();
+          layout_item_sort->set_full_field_details(field_primary_key);
+
+          found_set.m_sort_clause.clear();
+          found_set.m_sort_clause.push_back( type_pair_sort_field(layout_item_sort, true /* ascending */) );
+        }
+
+        m_Notebook_Data.init_db_details(found_set, primary_key_value_for_details);
         set_mode_widget(m_Notebook_Data);
 
         //Show how many records were found:
@@ -455,7 +470,7 @@ void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction
 
       //Check whether the current user has developer privileges:
       const ConnectionPool* connection_pool = ConnectionPool::get_instance();
-      bool test = get_user_is_in_group(connection_pool->get_user(), GLOM_STANDARD_GROUP_NAME_DEVELOPER);
+      bool test = Privs::get_user_is_in_group(connection_pool->get_user(), GLOM_STANDARD_GROUP_NAME_DEVELOPER);
       if(test)
       {
         std::cout << "DEBUG: User=" << connection_pool->get_user() << " _is_ in the developer group on the server." << std::endl;
@@ -512,7 +527,7 @@ void Frame_Glom::on_menu_file_export()
   g_assert(pWindowApp);
 
   //Do not try to export the data if the user may not view it:
-  Privileges table_privs = get_current_privs(m_table_name);
+  Privileges table_privs = Privs::get_current_privs(m_table_name);
   if(!table_privs.m_view)
   {
     show_ok_dialog(_("Export Not Allowed."), _("You do not have permission to view the data in this table, so you may not export the data."), *pWindowApp, Gtk::MESSAGE_ERROR);
@@ -536,7 +551,7 @@ void Frame_Glom::on_menu_file_export()
 
   //const int index_primary_key = fieldsSequence.size() - 1;
 
-  const Glib::ustring found_set_where_clause = m_Notebook_Data.get_where_clause();
+  const FoundSet found_set = m_Notebook_Data.get_found_set();
 
   std::fstream the_stream(filepath.c_str(), std::ios_base::out | std::ios_base::trunc);
   if(!the_stream)
@@ -545,20 +560,20 @@ void Frame_Glom::on_menu_file_export()
     return;
   }
 
-  export_data_to_stream(the_stream, m_table_name, mapGroupSequence, found_set_where_clause);
+  export_data_to_stream(the_stream, found_set, mapGroupSequence);
 }
 
-void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const Glib::ustring table_name, const Document_Glom::type_mapLayoutGroupSequence& sequence, const Glib::ustring& where_clause)
+void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const FoundSet& found_set, const Document_Glom::type_mapLayoutGroupSequence& sequence)
 {
-  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(table_name, sequence);
+  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
 
   if(fieldsSequence.empty())
     return;
 
-  const Glib::ustring query = GlomUtils::build_sql_select_with_where_clause(table_name, fieldsSequence, where_clause);
+  const Glib::ustring query = Utils::build_sql_select_with_where_clause(found_set.m_table_name, fieldsSequence, found_set.m_where_clause, found_set.m_sort_clause);
 
   //TODO: Lock the database (prevent changes) during export.
-  Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+  Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute(query, get_app_window());
 
   guint rows_count = 0;
   if(result)
@@ -597,17 +612,17 @@ void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const Glib::us
   }
 }
 
-void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const Glib::ustring table_name, const Document_Glom::type_mapLayoutGroupSequence& sequence, const Glib::ustring& where_clause)
+void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet& found_set, const Document_Glom::type_mapLayoutGroupSequence& sequence)
 {
-  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(table_name, sequence);
+  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
 
   if(fieldsSequence.empty())
     return;
 
-  const Glib::ustring query = GlomUtils::build_sql_select_with_where_clause(table_name, fieldsSequence, where_clause);
+  const Glib::ustring query = Utils::build_sql_select_with_where_clause(found_set.m_table_name, fieldsSequence, found_set.m_where_clause, found_set.m_sort_clause);
 
   //TODO: Lock the database (prevent changes) during export.
-  Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+  Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute(query, get_app_window());
 
   guint rows_count = 0;
   if(result)
@@ -636,7 +651,7 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const Glib::ust
 
             if(layout_item->get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
             {
-              if(!GlomConversions::value_is_empty(value))
+              if(!Conversions::value_is_empty(value))
                 std::cout << "  field name=" << layout_item->get_name() << ", image value not empty=" << std::endl;
             }
             //std::cout << "  field name=" << layout_item->get_name() << ", value=" << layout_item->m_field.sql(value) << std::endl;
@@ -664,6 +679,10 @@ void Frame_Glom::on_menu_Mode_Data()
 
 void Frame_Glom::on_menu_Mode_Find()
 {
+  //This can take quite a long time, flicking between 1 or 2 intermediate screens. 
+  //It shouldn't, but until we fix that, let's show the busy cursor while it's working:
+  Bakery::BusyCursor busy_cursor(get_app_window());
+
   const bool previously_in_data_mode = (m_Mode == MODE_Data);
 
   const Notebook_Data::dataview list_or_details = m_Notebook_Data.get_current_view();
@@ -722,6 +741,139 @@ void Frame_Glom::on_menu_Tables_EditTables()
   do_menu_Navigate_Table(); 
 }
 
+void Frame_Glom::on_menu_Tables_AddRelatedTable()
+{
+  //Delete and recreate the dialog,
+  //so we start with a blank one:
+  if(m_dialog_addrelatedtable)
+  {
+    remove_view(m_dialog_addrelatedtable);
+    delete m_dialog_addrelatedtable;
+    m_dialog_addrelatedtable = 0;
+  }
+
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_add_related_table");
+
+    refXml->get_widget_derived("dialog_add_related_table", m_dialog_addrelatedtable);
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+
+  if(!m_dialog_addrelatedtable)
+    return;
+
+  add_view(m_dialog_addrelatedtable); //Give it access to the document.
+  m_dialog_addrelatedtable->set_fields(m_table_name);
+
+  m_dialog_addrelatedtable->signal_request_edit_fields().connect( sigc::mem_fun(*this, &Frame_Glom::on_dialog_add_related_table_request_edit_fields) );
+
+  m_dialog_addrelatedtable->signal_response().connect( sigc::mem_fun(*this, &Frame_Glom::on_dialog_add_related_table_response) );
+
+  Gtk::Window* parent = get_app_window();
+
+  if(parent)
+    m_dialog_addrelatedtable->set_transient_for(*parent);
+
+  m_dialog_addrelatedtable->set_modal(); //We don't want people to edit the main window while we are changing structure.
+  m_dialog_addrelatedtable->show();
+}
+
+void Frame_Glom::on_dialog_add_related_table_response(int response)
+{
+  if(!m_dialog_addrelatedtable)
+    return;
+
+  m_dialog_addrelatedtable->hide();
+
+  bool stop_trying = false;
+  if(response == Gtk::RESPONSE_OK)
+  {
+    Glib::ustring table_name, relationship_name, from_key_name;
+    m_dialog_addrelatedtable->get_input(table_name, relationship_name, from_key_name);
+
+    Gtk::Window* parent = get_app_window();
+
+    //It would be nice to put this in the dialog's on_response() instead,
+    //but I don't think we can stop the response from being returned. murrayc
+    if(get_table_exists_in_database(table_name))
+    {
+      Frame_Glom::show_ok_dialog(_("Table Exists Already"), _("A table with this name already exists in the database. Please choose a different table name."), *parent, Gtk::MESSAGE_ERROR);
+    }
+    else if(get_relationship_exists(m_table_name, relationship_name))
+    {
+      Frame_Glom::show_ok_dialog(_("Relationship Exists Already"), _("A relationship with this name already exists for this table. Please choose a different relationship name."), *parent, Gtk::MESSAGE_ERROR);
+    }
+    else if(table_name.empty() || relationship_name.empty() || relationship_name.empty())
+    {
+      Frame_Glom::show_ok_dialog(_("More information needed"), _("You must specify a field, a table name, and a relationship name."), *parent, Gtk::MESSAGE_ERROR);
+    }
+    else
+    {
+      stop_trying = true;
+    }
+
+    if(!stop_trying)
+    {
+      //Offer the dialog again:
+      //This signal handler should be called again when a button is clicked.
+      m_dialog_addrelatedtable->show();
+    }
+    else
+    {
+      //Create the new table:
+      const bool result = create_table_with_default_fields(table_name);
+      if(!result)
+      {
+        std::cerr << "Frame_Glom::on_menu_Tables_AddRelatedTable(): create_table_with_default_fields() failed." << std::endl;
+        return;
+      }
+   
+      //Create the new relationship: 
+      sharedptr<Relationship> relationship = sharedptr<Relationship>::create();
+
+      relationship->set_name(relationship_name);
+      relationship->set_title(Utils::title_from_string(relationship_name));
+      relationship->set_from_table(m_table_name);
+      relationship->set_from_field(from_key_name);
+      relationship->set_to_table(table_name);
+
+      sharedptr<Field> related_primary_key = get_field_primary_key_for_table(table_name); //This field was created by create_table_with_default_fields().
+      if(!related_primary_key)
+      {
+        std::cerr << "Frame_Glom::on_menu_Tables_AddRelatedTable(): get_field_primary_key_for_table() failed." << std::endl;
+        return;
+      }
+
+      relationship->set_to_field(related_primary_key->get_name());
+
+      relationship->set_allow_edit(true);
+      relationship->set_auto_create(true);
+
+      Document_Glom* document = get_document();
+      if(!document)
+        return;
+
+      document->set_relationship(m_table_name, relationship);
+
+      on_dialog_tables_hide(); //Update the menu.
+
+      Gtk::Window* parent = get_app_window();
+      if(parent)
+        show_ok_dialog(_("Related Table Created"), _("The new related table has been created."), *parent, Gtk::MESSAGE_INFO);
+    }
+  }
+}
+
+void Frame_Glom::on_dialog_add_related_table_request_edit_fields()
+{
+  if(m_dialog_addrelatedtable)
+    do_menu_developer_fields(*m_dialog_addrelatedtable);
+}
+
 void Frame_Glom::do_menu_Navigate_Table(bool open_default)
 {
   if(get_document()->get_connection_database().empty())
@@ -816,14 +968,23 @@ void Frame_Glom::on_notebook_find_criteria(const Glib::ustring& where_clause)
   App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
   if(pApp)
   {
-    pApp->set_mode_data();
+    bool records_found = false;
 
-    //std::cout << "Frame_Glom::on_notebook_find_criteria: where_clause=" << where_clause << std::endl;
-    const bool records_found = m_Notebook_Data.init_db_details(m_table_name, where_clause);
+    { //Extra scope, to control the lifetime of the busy cursor. 
+      Bakery::BusyCursor busy_cursor(pApp);
 
-    //std::cout << "Frame_Glom::on_notebook_find_criteria(): BEFORE  m_Notebook_Data.select_page_for_find_results()" << std::endl;
-    m_Notebook_Data.select_page_for_find_results();
-    //std::cout << "Frame_Glom::on_notebook_find_criteria(): AFTER  m_Notebook_Data.select_page_for_find_results()" << std::endl;
+      pApp->set_mode_data();
+
+      //std::cout << "Frame_Glom::on_notebook_find_criteria: where_clause=" << where_clause << std::endl;
+      FoundSet found_set;
+      found_set.m_table_name = m_table_name;
+      found_set.m_where_clause = where_clause;
+      records_found = m_Notebook_Data.init_db_details(found_set);
+
+      //std::cout << "Frame_Glom::on_notebook_find_criteria(): BEFORE  m_Notebook_Data.select_page_for_find_results()" << std::endl;
+      m_Notebook_Data.select_page_for_find_results();
+      //std::cout << "Frame_Glom::on_notebook_find_criteria(): AFTER  m_Notebook_Data.select_page_for_find_results()" << std::endl;
+    }
 
     if(!records_found)
     {
@@ -1030,6 +1191,14 @@ void Frame_Glom::on_menu_developer_database_preferences()
 }
 
 void Frame_Glom::on_menu_developer_fields()
+{
+  Gtk::Window* parent = get_app_window();
+  if(parent)
+    do_menu_developer_fields(*parent);
+
+}
+
+void Frame_Glom::do_menu_developer_fields(Gtk::Window& parent)
 {
   //Check that there is a table to show:
   if(m_table_name.empty())
@@ -1038,7 +1207,24 @@ void Frame_Glom::on_menu_developer_fields()
   }
   else
   {
-    m_pDialog_Fields->set_transient_for(*get_app_window());
+    if(!m_pDialog_Fields)
+    {
+       try
+       {
+         Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_design");
+
+         refXml->get_widget_derived("window_design", m_pDialog_Fields);
+         m_pDialog_Fields->signal_hide().connect( sigc::mem_fun(*this, &Frame_Glom::on_developer_dialog_hide));
+       }
+       catch(const Gnome::Glade::XmlError& ex)
+       {
+         std::cerr << ex.what() << std::endl;
+       }
+
+       add_view(m_pDialog_Fields);
+    }
+
+    m_pDialog_Fields->set_transient_for(parent);
     m_pDialog_Fields->init_db_details(m_table_name);
     m_pDialog_Fields->show();
   }
@@ -1164,6 +1350,10 @@ void Frame_Glom::on_developer_dialog_hide()
 {
   //The dababase structure might have changed, so refresh the data view:
   show_table(m_table_name);
+
+  //TODO: This is a bit of a hack. It's not always useful to do this:
+  if(m_dialog_addrelatedtable)
+    m_dialog_addrelatedtable->set_fields(m_table_name);
 }
 
 bool Frame_Glom::connection_request_password_and_choose_new_database_name()
@@ -1227,7 +1417,16 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
           std::cout << "Frame_Glom::connection_request_password_and_choose_new_database_name(): unused database name successfully found: " << database_name_possible << std::endl; 
           //The connection to the server is OK, but the specified database does not exist.
           //That's good - we were looking for an unused database name.
-          get_document()->set_connection_database(database_name_possible);
+          Document_Glom* document = get_document();
+          if(document)
+          {
+            document->set_connection_database(database_name_possible);
+
+            ConnectionPool* connection_pool = ConnectionPool::get_instance();
+            if(connection_pool)
+              document->set_connection_server(connection_pool->get_host());
+          }
+
           return true;
         }
       }
@@ -1335,14 +1534,45 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib:
       return false;
     }
 
+
     if(sharedconnection)
     {
-      Bakery::BusyCursor(*get_app_window());
+      Gtk::Window* pWindowApp = get_app_window();
+      g_assert(pWindowApp);
+
+      Bakery::BusyCursor busycursor(*pWindowApp);
+
+      //std::cout << "Frame_Glom::create_database():  debug: before calling sharedconnection->get_gda_connection." << std::endl;
 
       Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
       if(connection)
       {
-        connection->create_database(database_name);
+        const bool test = connection->create_database(database_name);
+        if(!test)
+        {
+          //TODO: Discover the cause of the error somehow.
+          //I think a failure here might be caused by installing unstable libgda, which seems to affect stable libgda-1.2.
+          //Doing a "make install" in libgda-1.2 seems to fix this:
+          std::cerr << "Frame_Glom::create_database():  Gnome::Gda::Connection::create_database(" << database_name << ") failed." << std::endl;
+
+          //Tell the user:
+          Gtk::Dialog* dialog = 0;
+          try
+          {
+            Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_error_create_database");
+            refXml->get_widget("dialog_error_create_database", dialog);
+            dialog->set_transient_for(*pWindowApp);
+            dialog->run();
+            delete dialog;
+          }
+          catch(const Gnome::Glade::XmlError& ex)
+          {
+            std::cerr << ex.what() << std::endl;
+          }
+
+           return false;
+        }
+
         //if(result)
         //{
         //  std::cout << "Frame_Glom::create_database(): Creation succeeded: database_name=" << database_name << std::endl;
@@ -1368,11 +1598,15 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib:
 
   if(sharedconnection)
   {
-    add_standard_tables(); //Add internal, hidden, tables.
+    bool test = add_standard_tables(); //Add internal, hidden, tables.
+    if(!test)
+      return false;
 
     //Create the developer group, and make this user a member of it:
     //If we got this far then the user must really have developer privileges already:
-    add_standard_groups();
+    test = add_standard_groups();
+    if(!test)
+      return false;
 
     //std::cout << "Frame_Glom::create_database(): Creation of standard tables and groups finished." << std::endl;
 
@@ -1401,7 +1635,7 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib:
 
 void Frame_Glom::on_menu_report_selected(const Glib::ustring& report_name)
 {
-  const Privileges table_privs = get_current_privs(m_table_name);
+  const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
   //Don't try to print tables that the user can't view.
   if(!table_privs.m_view)
@@ -1410,11 +1644,16 @@ void Frame_Glom::on_menu_report_selected(const Glib::ustring& report_name)
     return;
   }
 
-  sharedptr<Report> report = get_document()->get_report(m_table_name, report_name);
+  Document_Glom* document = get_document();
+  sharedptr<Report> report = document->get_report(m_table_name, report_name);
   if(!report)
     return;
 
-  report_build(m_table_name, report, m_Notebook_Data.get_where_clause(), get_app_window()); //TODO: Use found set's where_clause.
+  FoundSet found_set = m_Notebook_Data.get_found_set();
+
+  ReportBuilder report_builder;
+  report_builder.set_document(document);
+  report_builder.report_build(found_set, report, get_app_window()); //TODO: Use found set's where_clause.
 }
 
 void Frame_Glom::on_dialog_layout_report_hide()
@@ -1473,8 +1712,8 @@ void Frame_Glom::on_dialog_tables_hide()
 
 void Frame_Glom::on_notebook_data_record_details_requested(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value)
 {
-  show_table(table_name);
-  m_Notebook_Data.show_details(primary_key_value);
+  show_table(table_name, primary_key_value);
+  //m_Notebook_Data.show_details(primary_key_value);
 }
 
 void Frame_Glom::update_records_count()
@@ -1522,3 +1761,5 @@ void Frame_Glom::on_button_find_all()
   show_table(m_table_name);
 }
 
+} //namespace Glom
+
