@@ -23,6 +23,9 @@
 
 #include "dialog_layout.h"
 
+namespace Glom
+{
+
 class Dialog_Layout_List : public Dialog_Layout
 {
 public:
@@ -84,4 +87,6 @@ protected:
   Gtk::Label* m_label_table_name;
 };
 
+} //namespace Glom
+
 #endif //GLOM_MODE_DATA_DIALOG_LAYOUT_LIST_H
