@@ -28,6 +28,9 @@
 //#include <libgnome/gnome-i18n.h>
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 Dialog_FieldCalculation::Dialog_FieldCalculation(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
 : Gtk::Dialog(cobject)
 {
@@ -84,7 +87,7 @@ void Dialog_FieldCalculation::on_button_test()
     for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
     {
       const sharedptr<const Field> field = *iter;
-      const Gnome::Gda::Value example_value = GlomConversions::get_example_value(field->get_glom_type());
+      const Gnome::Gda::Value example_value = Conversions::get_example_value(field->get_glom_type());
       field_values[field->get_name()] = example_value;
     }
   }
@@ -118,6 +121,7 @@ void Dialog_FieldCalculation::on_button_test()
   m_label_triggered_by->set_text(field_names);
 }
 
+} //namespace Glom
 
 
 
