@@ -19,10 +19,10 @@
  */
 
 #include "comboentryglom.h"
-#include "../data_structure/glomconversions.h"
+#include <glom/libglom/data_structure/glomconversions.h>
 #include <gtkmm/messagedialog.h>
 #include "../dialog_invalid_data.h"
-#include "../data_structure/glomconversions.h"
+#include <glom/libglom/data_structure/glomconversions.h>
 #include "../application.h"
 #include <glibmm/i18n.h>
 //#include <sstream> //For stringstream
@@ -31,6 +31,9 @@
 #include <ctime>     // for struct tm
 #include <iostream>   // for cout, endl
 
+namespace Glom
+{
+
 ComboEntryGlom::ComboEntryGlom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& /* refGlade */)
 : Gtk::ComboBoxEntry(cobject)
 {
@@ -121,7 +124,7 @@ void ComboEntryGlom::check_for_change()
     bool success = false;
 
     sharedptr<const LayoutItem_Field> layout_item = sharedptr<const LayoutItem_Field>::cast_dynamic(get_layout_item());
-    Gnome::Gda::Value value = GlomConversions::parse_value(layout_item->get_glom_type(), get_entry()->get_text(), layout_item->get_formatting_used().m_numeric_format, success);
+    Gnome::Gda::Value value = Conversions::parse_value(layout_item->get_glom_type(), get_entry()->get_text(), layout_item->get_formatting_used().m_numeric_format, success);
 
     if(success)
     {
@@ -177,7 +180,7 @@ void ComboEntryGlom::set_value(const Gnome::Gda::Value& value)
   sharedptr<const LayoutItem_Field> layout_item = sharedptr<const LayoutItem_Field>::cast_dynamic(get_layout_item());
   if(layout_item)
   {
-    set_text(GlomConversions::get_text_for_gda_value(layout_item->get_glom_type(), value, layout_item->get_formatting_used().m_numeric_format));
+    set_text(Conversions::get_text_for_gda_value(layout_item->get_glom_type(), value, layout_item->get_formatting_used().m_numeric_format));
   }
 }
 
@@ -194,7 +197,7 @@ Gnome::Gda::Value ComboEntryGlom::get_value() const
   bool success = false;
 
   sharedptr<const LayoutItem_Field> layout_item = sharedptr<const LayoutItem_Field>::cast_dynamic(get_layout_item());
-  return GlomConversions::parse_value(layout_item->get_glom_type(), get_entry()->get_text(), layout_item->get_formatting_used().m_numeric_format, success);
+  return Conversions::parse_value(layout_item->get_glom_type(), get_entry()->get_text(), layout_item->get_formatting_used().m_numeric_format, success);
 }
 
 bool ComboEntryGlom::on_entry_button_press_event(GdkEventButton *event)
@@ -262,3 +265,5 @@ void ComboEntryGlom::set_read_only(bool read_only)
     entry->set_editable(!read_only);
 }
 
+
+} //namespace Glom
